@@ -1,5 +1,5 @@
 /*
- * $Id: gfhost.c 4724 2010-06-12 15:29:45Z tatebe $
+ * $Id: gfhost.c 4831 2010-07-17 13:37:18Z tatebe $
  */
 
 #include <assert.h>
@@ -266,9 +266,12 @@ parse_string_long(char **linep, int lineno, const char *diag, long *retvalp)
 
 	line += strspn(line, space); /* skip space */
 	len = strcspn(line, space);
-	if (len == 0 || line[len] == '\0')
+	if (len == 0)
 		return (invalid_input(lineno));
-	line[len] = '\0';
+	else if (line[len] != '\0') {
+		line[len] = '\0';
+		++len;
+	}
 	errno = 0;
 	retval = strtol(line, &s, 0);
 	if (s == line) {
@@ -282,7 +285,7 @@ parse_string_long(char **linep, int lineno, const char *diag, long *retvalp)
 		    lineno, strerror(errno), line);
 		return (GFARM_ERR_INVALID_ARGUMENT);
 	}
-	line += len + 1;
+	line += len;
 	*linep = line;
 	*retvalp = retval;
 	return (GFARM_ERR_NO_ERROR);
