@@ -1,5 +1,5 @@
 /*
- * $Id: gfmkdir.c 2640 2006-05-29 22:38:29Z soda $
+ * $Id: gfmkdir.c 4952 2010-12-14 16:09:31Z tatebe $
  */
 
 #include <stdio.h>
@@ -14,16 +14,42 @@ char *program_name = "gfmkdir";
 static void
 usage(void)
 {
-	fprintf(stderr, "Usage: %s directory...\n", program_name);
+	fprintf(stderr, "Usage: %s [-p] directory...\n", program_name);
 	exit(1);
 }
 
+static gfarm_error_t
+gfs_mkdir_p(char *path, gfarm_mode_t mode, int option_parent)
+{
+	gfarm_error_t e;
+	struct gfs_stat sb;
+
+	if (option_parent) {
+		char *parent = gfarm_url_dir(path);
+
+		if (parent == NULL)
+			return (GFARM_ERR_NO_MEMORY);
+		e = gfs_stat(parent, &sb);
+		if (e == GFARM_ERR_NO_ERROR)
+			gfs_stat_free(&sb);
+		else if (e == GFARM_ERR_NO_SUCH_FILE_OR_DIRECTORY)
+			e = gfs_mkdir_p(parent, mode, option_parent);
+		free(parent);
+		if (e != GFARM_ERR_NO_ERROR)
+			return (e);
+	}
+	e = gfs_mkdir(path, mode);
+	if (option_parent && e == GFARM_ERR_ALREADY_EXISTS)
+		e = GFARM_ERR_NO_ERROR;
+	return (e);
+}
+
 int
 main(int argc, char **argv)
 {
 	gfarm_error_t e;
 	int i, c, status = 0;
-	extern int optind;
+	int option_parent = 0;
 
 	if (argc > 0)
 		program_name = basename(argv[0]);
@@ -34,8 +60,11 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	while ((c = getopt(argc, argv, "h?")) != -1) {
+	while ((c = getopt(argc, argv, "hp?")) != -1) {
 		switch (c) {
+		case 'p':
+			option_parent = 1;
+			break;
 		case 'h':
 		case '?':
 		default:
@@ -48,7 +77,7 @@ main(int argc, char **argv)
 		usage();
 
 	for (i = 0; i < argc; i++) {
-		e = gfs_mkdir(argv[i], 0755);
+		e = gfs_mkdir_p(argv[i], 0755, option_parent);
 		if (e != GFARM_ERR_NO_ERROR) {
 			fprintf(stderr, "%s: %s: %s\n",
 			    program_name, argv[i], gfarm_error_string(e));
