@@ -41,7 +41,6 @@ void
 display_replica(struct gfs_stat *st, struct gfs_replica_info *ri, int i,
 	struct options *opt)
 {
-	gfarm_uint64_t igen;
 	int need_space;
 
 	if (opt->long_format)
@@ -53,8 +52,9 @@ display_replica(struct gfs_stat *st, struct gfs_replica_info *ri, int i,
 
 	if (opt->type_suffix) {
 		if (opt->print_dead_copy &&
-		    (igen = gfs_replica_info_nth_gen(ri, i)) != st->st_gen)
-			printf(";%llu", (unsigned long long)igen);
+		    gfs_replica_info_nth_is_dead_copy(ri, i))
+			printf(";%llu",
+			    (long long)gfs_replica_info_nth_gen(ri, i));
 		if (gfs_replica_info_nth_is_incomplete(ri, i))
 			putchar('?');
 		if (gfs_replica_info_nth_is_dead_host(ri, i))
@@ -85,7 +85,7 @@ display_replica(struct gfs_stat *st, struct gfs_replica_info *ri, int i,
 		if (opt->print_dead_copy) {
 			if (need_space)
 				putchar(' ');
-			putchar(gfs_replica_info_nth_gen(ri, i) != st->st_gen ?
+			putchar(gfs_replica_info_nth_is_dead_copy(ri, i) ?
 			    'o' : '-');
 		}
 		putchar('\n');
