@@ -1,5 +1,5 @@
 /*
- * $Id: config.c 4640 2010-04-12 12:34:22Z n-soda $
+ * $Id: config.c 5018 2010-12-30 08:35:34Z tatebe $
  */
 
 #include <sys/types.h>
@@ -26,6 +26,7 @@
 #include "gfutil.h"
 
 #include "liberror.h"
+#include "patmatch.h"
 #include "hostspec.h"
 #include "param.h"
 #include "sockopt.h"
@@ -48,6 +49,68 @@ gfarm_config_set_filename(char *filename)
 /* XXX move actual function definition here */
 static gfarm_error_t gfarm_strtoken(char **, char **);
 
+/*
+ * xattr cache handling
+ */
+static gfarm_stringlist xattr_cache_list;
+
+/*
+ * NOTE:
+ * client host should call gfs_stat_cache_clear() after
+ * calling this gfarm_xattr_caching_pattern_add() function,
+ * otherwise unexpected GFARM_ERR_NO_SUCH_OBJECT may happen.
+ *
+ * The reason we don't call gfs_stat_cache_clear() automatically is
+ * because it's not appropriate for gfmd.
+ */
+gfarm_error_t
+gfarm_xattr_caching_pattern_add(const char *attr_pattern)
+{
+	gfarm_error_t e;
+	char *pat = strdup(attr_pattern);
+
+	if (pat == NULL) {
+		gflog_debug(GFARM_MSG_1002446,
+		    "failed to allocate an attr_pattern \"%s\": no memory",
+		    attr_pattern);
+		return (GFARM_ERR_NO_MEMORY);
+	}
+	e = gfarm_stringlist_add(&xattr_cache_list, pat);
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1002447,
+		    "failed to allocate record an attr_pattern \"%s\": "
+		    "no memory", attr_pattern);
+		free(pat);
+	}
+	return (e);
+}
+
+int
+gfarm_xattr_caching(const char *attrname)
+{
+	int i, n = gfarm_stringlist_length(&xattr_cache_list);
+	const char *pattern;
+
+	for (i = 0; i < n; i++) {
+		pattern = gfarm_stringlist_elem(&xattr_cache_list, i);
+		if (gfarm_pattern_match(pattern, attrname, 0))
+			return (1);
+	}
+	return (0);
+}
+
+int
+gfarm_xattr_caching_patterns_number(void)
+{
+	return (gfarm_stringlist_length(&xattr_cache_list));
+}
+
+char **
+gfarm_xattr_caching_patterns(void)
+{
+	return (GFARM_STRINGLIST_STRARRAY(xattr_cache_list));
+}
+
 /*
  * GFarm username handling
  */
@@ -57,7 +120,7 @@ static gfarm_stringlist local_group_map_file_list;
 
 /* the return value of the following function should be free(3)ed */
 static gfarm_error_t
-map_user(gfarm_stringlist map_file_list, char *from, char **to_p,
+map_user(gfarm_stringlist *map_file_list, char *from, char **to_p,
 	char *(*mapping)(char *, char *, char *),
 	gfarm_error_t error_redefined)
 {
@@ -69,10 +132,10 @@ map_user(gfarm_stringlist map_file_list, char *from, char **to_p,
 	int lineno = 0;
 
 	*to_p = NULL;
-	list_len = gfarm_stringlist_length(&map_file_list);
+	list_len = gfarm_stringlist_length(map_file_list);
 	mapfile_mapped_index = list_len;
 	for (i = 0; i < list_len; i++) {
-		mapfile = gfarm_stringlist_elem(&map_file_list, i);
+		mapfile = gfarm_stringlist_elem(map_file_list, i);
 		if ((map = fopen(mapfile, "r")) == NULL) {
 			gflog_error(GFARM_MSG_1000009,
 			    "%s: cannot open: %s", mapfile, strerror(errno));
@@ -159,7 +222,7 @@ map_global_to_local(char *from, char *global_user, char *local_user)
 gfarm_error_t
 gfarm_global_to_local_username(char *global_user, char **local_user_p)
 {
-	return (map_user(local_user_map_file_list, global_user, local_user_p,
+	return (map_user(&local_user_map_file_list, global_user, local_user_p,
 	    map_global_to_local, GFARM_ERRMSG_GLOBAL_USER_REDEFIEND));
 }
 
@@ -181,7 +244,7 @@ map_local_to_global(char *from, char *global_user, char *local_user)
 gfarm_error_t
 gfarm_local_to_global_username(char *local_user, char **global_user_p)
 {
-	return (map_user(local_user_map_file_list, local_user, global_user_p,
+	return (map_user(&local_user_map_file_list, local_user, global_user_p,
 	    map_local_to_global, GFARM_ERRMSG_LOCAL_USER_REDEFIEND));
 }
 
@@ -195,7 +258,7 @@ gfarm_local_to_global_username(char *local_user, char **global_user_p)
 gfarm_error_t
 gfarm_global_to_local_groupname(char *global_user, char **local_user_p)
 {
-	return (map_user(local_group_map_file_list, global_user, local_user_p,
+	return (map_user(&local_group_map_file_list, global_user, local_user_p,
 	    map_global_to_local, GFARM_ERRMSG_GLOBAL_GROUP_REDEFIEND));
 }
 
@@ -209,7 +272,7 @@ gfarm_global_to_local_groupname(char *global_user, char **local_user_p)
 gfarm_error_t
 gfarm_local_to_global_groupname(char *local_user, char **global_user_p)
 {
-	return (map_user(local_group_map_file_list, local_user, global_user_p,
+	return (map_user(&local_group_map_file_list, local_user, global_user_p,
 	    map_local_to_global, GFARM_ERRMSG_LOCAL_GROUP_REDEFIEND));
 }
 
@@ -355,6 +418,8 @@ char *gfarm_localfs_datadir = NULL;
 
 /* miscellaneous */
 #define GFARM_LOG_MESSAGE_VERBOSE_DEFAULT	0
+#define GFARM_NO_FILE_SYSTEM_NODE_TIMEOUT_DEFAULT 30 /* 30 seconds */
+#define GFARM_GFMD_RECONNECTION_TIMEOUT_DEFAULT 30 /* 30 seconds */
 #define GFARM_ATTR_CACHE_LIMIT_DEFAULT		40000 /* 40,000 entries */
 #define GFARM_ATTR_CACHE_TIMEOUT_DEFAULT	1000 /* 1,000 milli second */
 #define GFARM_SCHEDULE_CACHE_TIMEOUT_DEFAULT 600 /* 10 minutes */
@@ -370,6 +435,8 @@ char *gfarm_localfs_datadir = NULL;
 #define MISC_DEFAULT -1
 int gfarm_log_level = MISC_DEFAULT;
 int gfarm_log_message_verbose = MISC_DEFAULT;
+int gfarm_no_file_system_node_timeout = MISC_DEFAULT;
+int gfarm_gfmd_reconnection_timeout = MISC_DEFAULT;
 int gfarm_attr_cache_limit = MISC_DEFAULT;
 int gfarm_attr_cache_timeout = MISC_DEFAULT;
 int gfarm_schedule_cache_timeout = MISC_DEFAULT;
@@ -937,67 +1004,93 @@ parse_address_use_arguments(char *p, char **op)
 #endif
 
 static gfarm_error_t
-parse_local_user_map(char *p, char **op)
+parse_known_network_arguments(char *p, char **op)
 {
 	gfarm_error_t e;
-	char *tmp, *mapfile;
+	char *tmp, *address;
+	struct gfarm_hostspec *hostspecp;
 
-	e = gfarm_strtoken(&p, &mapfile);
-	if (e != GFARM_ERR_NO_ERROR) {
-		gflog_debug(GFARM_MSG_1000950,
-			"parsing of local user map "
-			"mapfile argument (%s) failed: %s",
-			p, gfarm_error_string(e));
-		return (e);
-	}
-	if (mapfile == NULL) {
-		gflog_debug(GFARM_MSG_1000951,
-			"Missing local user map mapfile argument");
-		return (GFARM_ERRMSG_MISSING_USER_MAP_FILE_ARGUMENT);
-	}
-	e = gfarm_strtoken(&p, &tmp);
-	if (e != GFARM_ERR_NO_ERROR) {
-		gflog_debug(GFARM_MSG_1000952,
-			"parsing of local user map arguments (%s) failed: %s",
-			p, gfarm_error_string(e));
-		return (e);
-	}
-	if (tmp != NULL) {
-		gflog_debug(GFARM_MSG_1000953,
-			"Too many local user map arguments passed");
-		return (GFARM_ERRMSG_TOO_MANY_ARGUMENTS);
-	}
-	mapfile = strdup(mapfile);
-	if (mapfile == NULL) {
-		gflog_debug(GFARM_MSG_1000954,
-			"allocation of 'mapfile' failed: %s",
-			gfarm_error_string(GFARM_ERR_NO_MEMORY));
-		return (GFARM_ERR_NO_MEMORY);
-	}
-	e = gfarm_stringlist_add(&local_user_map_file_list, mapfile);
-	return (e);
-}
+	/* assert(strcmp(*op, "known_network") == 0); */
 
-static gfarm_error_t
-parse_local_group_map(char *p, char **op)
-{
-	gfarm_error_t e;
-	char *tmp, *mapfile;
-
-	e = gfarm_strtoken(&p, &mapfile);
+	e = gfarm_strtoken(&p, &address);
 	if (e != GFARM_ERR_NO_ERROR)
 		return (e);
-	if (mapfile == NULL)
-		return (GFARM_ERRMSG_MISSING_GROUP_MAP_FILE_ARGUMENT);
+	if (address == NULL)
+		return (GFARM_ERRMSG_MISSING_ADDRESS_ARGUMENT);
 	e = gfarm_strtoken(&p, &tmp);
 	if (e != GFARM_ERR_NO_ERROR)
 		return (e);
 	if (tmp != NULL)
 		return (GFARM_ERRMSG_TOO_MANY_ARGUMENTS);
-	mapfile = strdup(mapfile);
-	if (mapfile == NULL)
+
+	e = gfarm_hostspec_parse(address, &hostspecp);
+	if (e != GFARM_ERR_NO_ERROR) {
+		/*
+		 * we don't return `host' to *op here,
+		 * because it may be too long.
+		 */
+		*op = "1st(address) argument";
+		return (e);
+	}
+
+	e = gfarm_known_network_list_add(hostspecp);
+	if (e != GFARM_ERR_NO_ERROR) {
+		/*
+		 * we don't return `option' to *op here,
+		 * because it may be too long.
+		 */
+		*op = "1st(address) argument";
+		gfarm_hostspec_free(hostspecp);
+		return (e);
+	}
+	return (GFARM_ERR_NO_ERROR);
+}
+
+static gfarm_error_t
+parse_stringlist(char *p, char **op,
+	gfarm_stringlist *list, const char *listname)
+{
+	gfarm_error_t e;
+	char *tmp, *arg;
+
+	e = gfarm_strtoken(&p, &arg);
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1002448,
+		    "parsing argument %s of %s failed: %s",
+		    p, listname, gfarm_error_string(e));
+		return (e);
+	}
+	if (arg == NULL) {
+		gflog_debug(GFARM_MSG_1002449,
+		    "missing argument for %s", listname);
+		return (GFARM_ERRMSG_MISSING_USER_MAP_FILE_ARGUMENT);
+	}
+	e = gfarm_strtoken(&p, &tmp);
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1002450,
+		    "parsing argument %s of %s failed: %s",
+		    p, listname, gfarm_error_string(e));
+		return (e);
+	}
+	if (tmp != NULL) {
+		gflog_debug(GFARM_MSG_1002451,
+		    "Too many arguments for %s", listname);
+		return (GFARM_ERRMSG_TOO_MANY_ARGUMENTS);
+	}
+	arg = strdup(arg);
+	if (arg == NULL) {
+		gflog_debug(GFARM_MSG_1002452,
+		    "failed to allocate an argument of %s: no memory",
+		    listname);
 		return (GFARM_ERR_NO_MEMORY);
-	e = gfarm_stringlist_add(&local_group_map_file_list, mapfile);
+	}
+	e = gfarm_stringlist_add(list, arg);
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1002453,
+		    "failed to allocate a %s entry for \"%s\": no memory",
+		    listname, arg);
+		free(arg);
+	}
 	return (e);
 }
 
@@ -1422,10 +1515,17 @@ parse_one_line(char *s, char *p, char **op)
 	} else if (strcmp(s, o = "address_use") == 0) {
 		e = parse_address_use_arguments(p, &o);
 #endif
+	} else if (strcmp(s, o = "known_network") == 0) {
+		e = parse_known_network_arguments(p, &o);
+	} else if (strcmp(s, o = "xattr_cache") == 0) {
+		e = parse_stringlist(p, &o,
+		    &xattr_cache_list, "xattr cache");
 	} else if (strcmp(s, o = "local_user_map") == 0) {
-		e = parse_local_user_map(p, &o);
+		e = parse_stringlist(p, &o,
+		    &local_user_map_file_list, "local user map");
 	} else if (strcmp(s, o = "local_group_map") == 0) {
-		e = parse_local_group_map(p, &o);
+		e = parse_stringlist(p, &o,
+		    &local_group_map_file_list, "local group map");
 #if 0 /* XXX NOTYET */
 	} else if (strcmp(s, o = "client_architecture") == 0) {
 		e = parse_client_architecture(p, &o);
@@ -1436,6 +1536,10 @@ parse_one_line(char *s, char *p, char **op)
 	} else if (strcmp(s, o = "log_message_verbose_level") == 0) {
 		e = parse_set_misc_int(p, &gfarm_log_message_verbose);
 		gflog_set_message_verbose(gfarm_log_message_verbose);
+	} else if (strcmp(s, o = "no_file_system_node_timeout") == 0) {
+		e = parse_set_misc_int(p, &gfarm_no_file_system_node_timeout);
+	} else if (strcmp(s, o = "gfmd_reconnection_timeout") == 0) {
+		e = parse_set_misc_int(p, &gfarm_gfmd_reconnection_timeout);
 	} else if (strcmp(s, o = "attr_cache_limit") == 0) {
 		e = parse_set_misc_int(p, &gfarm_attr_cache_limit);
 	} else if (strcmp(s, o = "attr_cache_timeout") == 0) {
@@ -1487,29 +1591,21 @@ parse_one_line(char *s, char *p, char **op)
 }
 
 gfarm_error_t
-gfarm_init_user_map(void)
+gfarm_init_config_stringlists(void)
 {
-	gfarm_stringlist_init(&local_user_map_file_list);
-	return (GFARM_ERR_NO_ERROR);
-}
+	gfarm_stringlist_init(&xattr_cache_list);
 
-gfarm_error_t
-gfarm_init_group_map(void)
-{
+	gfarm_stringlist_init(&local_user_map_file_list);
 	gfarm_stringlist_init(&local_group_map_file_list);
 	return (GFARM_ERR_NO_ERROR);
 }
 
 gfarm_error_t
-gfarm_free_user_map(void)
+gfarm_free_config_stringlists(void)
 {
-	gfarm_stringlist_free_deeply(&local_user_map_file_list);
-	return (GFARM_ERR_NO_ERROR);
-}
+	gfarm_stringlist_free_deeply(&xattr_cache_list);
 
-gfarm_error_t
-gfarm_free_group_map(void)
-{
+	gfarm_stringlist_free_deeply(&local_user_map_file_list);
 	gfarm_stringlist_free_deeply(&local_group_map_file_list);
 	return (GFARM_ERR_NO_ERROR);
 }
@@ -1578,6 +1674,12 @@ gfarm_config_set_default_misc(void)
 		gfarm_log_message_verbose = GFARM_LOG_MESSAGE_VERBOSE_DEFAULT;
 	gflog_set_message_verbose(gfarm_log_message_verbose);
 
+	if (gfarm_no_file_system_node_timeout == MISC_DEFAULT)
+		gfarm_no_file_system_node_timeout =
+		    GFARM_NO_FILE_SYSTEM_NODE_TIMEOUT_DEFAULT;
+	if (gfarm_gfmd_reconnection_timeout == MISC_DEFAULT)
+		gfarm_gfmd_reconnection_timeout =
+		    GFARM_GFMD_RECONNECTION_TIMEOUT_DEFAULT;
 	if (gfarm_attr_cache_limit == MISC_DEFAULT)
 		gfarm_attr_cache_limit = GFARM_ATTR_CACHE_LIMIT_DEFAULT;
 	if (gfarm_attr_cache_timeout == MISC_DEFAULT)
