@@ -115,8 +115,7 @@ gfarm_config_read(void)
 		sprintf(rc, "%s/%s", home, gfarm_client_rc);
 		rc_need_free = 1;
 	}
-	gfarm_init_user_map();
-	gfarm_init_group_map();
+	gfarm_init_config_stringlists();
 	if ((config = fopen(rc, "r")) == NULL) {
 		user_config_errno = errno;
 	} else {
@@ -523,8 +522,7 @@ gfarm_terminate(void)
 			return (e);
 	}
 #endif /* not yet in gfarm v2 */
-	gfarm_free_user_map();
-	gfarm_free_group_map();
+	gfarm_free_config_stringlists();
 	gfs_client_terminate();
 	gfm_client_terminate();
 	gflog_terminate();
