@@ -1,8 +1,9 @@
 /*
- * $Id: config_server.c 4457 2010-02-23 01:53:23Z ookuma $
+ * $Id: config_server.c 5006 2010-12-29 23:36:30Z n-soda $
  */
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 
 #include <gfarm/gfarm.h>
 
@@ -30,8 +31,7 @@ gfarm_server_config_read(void)
 	int lineno;
 	FILE *config;
 
-	gfarm_init_user_map();
-	gfarm_init_group_map();
+	gfarm_init_config_stringlists();
 	if ((config = fopen(gfarm_config_file, "r")) == NULL) {
 		gflog_debug(GFARM_MSG_1000976,
 			"open operation on server config file (%s) failed",
