@@ -30,8 +30,12 @@ gfs_dirplus_alloc(struct gfm_connection *gfm_server, gfarm_int32_t fd,
 	GFS_DirPlus dir;
 
 	GFARM_MALLOC(dir);
-	if (dir == NULL)
+	if (dir == NULL) {
+		gflog_debug(GFARM_MSG_1001277,
+			"allocation of dir failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
 		return (GFARM_ERR_NO_MEMORY);
+	}
 
 	dir->gfm_server = gfm_server;
 	dir->fd = fd;
@@ -59,8 +63,13 @@ gfs_opendirplus(const char *path, GFS_DirPlus *dirp)
 	int fd, type;
 
 	if ((e = gfm_open_fd(path, GFARM_FILE_RDONLY, &gfm_server, &fd, &type))
-	    != GFARM_ERR_NO_ERROR)
+	    != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001278,
+			"gfm_open_fd(%s) failed: %s",
+			path,
+			gfarm_error_string(e));
 		return (e);
+	}
 
 	if (type != GFS_DT_DIR)
 		e = GFARM_ERR_NOT_A_DIRECTORY;
@@ -68,6 +77,17 @@ gfs_opendirplus(const char *path, GFS_DirPlus *dirp)
 	    GFARM_ERR_NO_ERROR)
 		return (GFARM_ERR_NO_ERROR);
 
+	if (e == GFARM_ERR_NOT_A_DIRECTORY)
+		gflog_debug(GFARM_MSG_1001279,
+			"Not a directory (%s): %s",
+			path,
+			gfarm_error_string(e));
+	else if (e != GFARM_ERR_NO_ERROR)
+		gflog_debug(GFARM_MSG_1001280,
+			"allocation of dirplus for path (%s) failed: %s",
+			path,
+			gfarm_error_string(e));
+
 	(void)gfm_close_fd(gfm_server, fd); /* ignore result */
 	gfm_client_connection_free(gfm_server);
 	return (e);
@@ -114,8 +134,12 @@ gfs_readdirplus(GFS_DirPlus dir,
 		    gfm_getdirentsplus_result,
 		    NULL, 
 		    dir);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001281,
+				"gfs_client_compound_fd_op() failed: %s",
+				gfarm_error_string(e));
 			return (e);
+		}
 		if (dir->n == 0) {
 			*entry = NULL;
 			*status = NULL;
