@@ -2,6 +2,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <string.h>
 #include <openssl/evp.h>
 #include <gfarm/gfarm.h>
 #include "gfs_proto.h"
@@ -18,12 +19,16 @@ gfs_digest_calculate_local(int fd, char *buffer, size_t buffer_size,
 	size_t *md_lenp, unsigned char *md_value,
 	gfarm_off_t *filesizep)
 {
-	int size;
+	int size, save_errno;
 	gfarm_off_t off = 0;
 	unsigned int len;
 
-	if (lseek(fd, (off_t)0, 0) == -1)
-		return (errno);
+	if (lseek(fd, (off_t)0, 0) == -1) {
+		save_errno = errno;
+		gflog_debug(GFARM_MSG_1001020, "lseek() failed: %s",
+			strerror(save_errno));
+		return (save_errno);
+	}
 
 	EVP_DigestInit(md_ctx, md_type);
 	while ((size = read(fd, buffer, buffer_size)) > 0) {
@@ -34,5 +39,13 @@ gfs_digest_calculate_local(int fd, char *buffer, size_t buffer_size,
 
 	*md_lenp = len;
 	*filesizep = off;
-	return (size == -1 ? errno : 0);
+
+	if (size == -1) {
+		save_errno = errno;
+		gflog_debug(GFARM_MSG_1001021, "read() failed: %s",
+			strerror(save_errno));
+		return (save_errno);
+	}
+
+	return (0);
 }
