@@ -4,6 +4,7 @@
 
 #include <unistd.h>
 #include <gfarm/error.h>
+#include <gfarm/gflog.h>
 #include <gfarm/gfarm_misc.h>
 #include <gfarm/gfs.h>
 #include "gfm_client.h"
@@ -21,8 +22,13 @@ gfs_statfs(gfarm_off_t *used, gfarm_off_t *avail, gfarm_off_t *files)
 	for (;;) {
 		if ((e = gfm_client_connection_and_process_acquire(
 		    gfarm_metadb_server_name, gfarm_metadb_server_port,
-		    &gfm_server)) != GFARM_ERR_NO_ERROR)
+		    &gfm_server)) != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001379,
+				"acquirement of client connection and process "
+				"failed: %s",
+				gfarm_error_string(e));
 			return (e);
+		}
 
 		e = gfm_client_statfs(gfm_server, used, avail, files);
 		if (gfm_client_is_connection_error(e) && ++retry <= 1){
