@@ -1,5 +1,5 @@
 /*
- * $Id: gfs_unlink.c 4310 2010-01-12 10:14:59Z ookuma $
+ * $Id: gfs_unlink.c 4457 2010-02-23 01:53:23Z ookuma $
  */
 
 #include <stddef.h>
@@ -27,12 +27,22 @@ gfs_unlink(const char *path)
 	gfs_profile(gfarm_gettimerval(&t1));
 
 	e = gfs_stat(path, &st);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001396,
+			"gfs_stat(%s) failed: %s",
+			path,
+			gfarm_error_string(e));
 		return (e);
+	}
 	is_dir = GFARM_S_ISDIR(st.st_mode);
 	gfs_stat_free(&st);
-	if (is_dir)
+	if (is_dir) {
+		gflog_debug(GFARM_MSG_1001397,
+			"Not a directory(%s): %s",
+			path,
+			gfarm_error_string(GFARM_ERR_IS_A_DIRECTORY));
 		return (GFARM_ERR_IS_A_DIRECTORY);
+	}
 
 	/* XXX FIXME there is race condition here */
 
