@@ -1,5 +1,5 @@
 /*
- * $Id: liberror.c 4575 2010-03-25 13:53:58Z tatebe $
+ * $Id: liberror.c 4527 2010-03-12 10:14:44Z n-soda $
  */
 
 #include <pthread.h>
@@ -130,6 +130,7 @@ static const char *errcode_string[GFARM_ERR_NUMBER] = {
 	"hostname is missing in a Gfarm URL",
 	"port number is missing in a Gfarm URL",
 	"port number is invalid in a Gfarm URL",
+	"file busy",
 };
 
 static const char *errmsg_string[GFARM_ERRMSG_END - GFARM_ERRMSG_BEGIN] = {
@@ -408,6 +409,7 @@ static struct gfarm_errno_error_map {
 	/*		GFARM_ERR_GFARM_URL_HOST_IS_MISSING */
 	/*		GFARM_ERR_GFARM_URL_PORT_IS_MISSING */
 	/*		GFARM_ERR_GFARM_URL_PORT_IS_INVALID */
+	{ EBUSY,	GFARM_ERR_FILE_BUSY },
 };
 
 struct gfarm_error_domain {
@@ -443,6 +445,8 @@ gfarm_error_domain_alloc(int domerror_min, int domerror_max,
 	struct gfarm_error_domain *last, *new;
 
 	if (gfarm_error_domain_number >= MAX_ERROR_DOMAINS) {
+		gflog_debug(GFARM_MSG_1000845,
+		    "gfarm_error_domain_alloc: too many error domains");
 		return (GFARM_ERRMSG_TOO_MANY_ERROR_DOMAIN);
 	}
 	if (gfarm_error_domain_number == 0) {
@@ -453,7 +457,7 @@ gfarm_error_domain_alloc(int domerror_min, int domerror_max,
 	}
 	if (next_error + domerror_max - domerror_min
 	    >= GFARM_ERR_PRIVATE_BEGIN) {
-		gflog_debug(GFARM_MSG_UNFIXED,
+		gflog_debug(GFARM_MSG_1000846,
 		    "gfarm_error_domain_alloc: too large domain [%d, %d]",
 		    domerror_min, domerror_max);
 		return (GFARM_ERR_NUMERICAL_ARGUMENT_OUT_OF_DOMAIN);
@@ -490,7 +494,7 @@ gfarm_error_range_alloc(int domerror_min, int domerror_max,
 	/* sanity checks */
 	if (domerror_min < GFARM_ERR_PRIVATE_BEGIN ||
 	    domerror_max < domerror_min) {
-		gflog_debug(GFARM_MSG_UNFIXED,
+		gflog_debug(GFARM_MSG_1000847,
 		    "gfarm_error_range_alloc: invalid range [%d, %d]",
 		    domerror_min, domerror_max);
 		return (GFARM_ERR_NUMERICAL_ARGUMENT_OUT_OF_DOMAIN);
@@ -498,7 +502,7 @@ gfarm_error_range_alloc(int domerror_min, int domerror_max,
 	for (dom = gfarm_error_ranges; dom != NULL; dom = dom->next) {
 		if (domerror_max >= dom->offset &&
 		    domerror_min < dom->offset + dom->domerror_number - 1) {
-			gflog_debug(GFARM_MSG_UNFIXED,
+			gflog_debug(GFARM_MSG_1000848,
 			    "gfarm_error_range_alloc: [%d, %d] is "
 			    "overlapping with an existing range [%d, %d]",
 			    domerror_min, domerror_max, dom->offset,
@@ -509,7 +513,7 @@ gfarm_error_range_alloc(int domerror_min, int domerror_max,
 
 	GFARM_MALLOC(new);
 	if (new == NULL) {
-		gflog_debug(GFARM_MSG_UNFIXED,
+		gflog_debug(GFARM_MSG_1000849,
 		    "gfarm_error_range_alloc: no memory");
 		return (GFARM_ERR_NO_MEMORY);
 	}
@@ -533,7 +537,7 @@ gfarm_error_domain_add_map(struct gfarm_error_domain *domain,
 
 	if (domerror < domain->domerror_min ||
 	    domerror >= domain->domerror_min + domain->domerror_number) {
-		gflog_debug(GFARM_MSG_UNFIXED,
+		gflog_debug(GFARM_MSG_1000850,
 		    "gfarm_error_domain_add_map: "
 		    "error code %d is out of [%d, %d]",
 		    domerror, domain->domerror_min,
@@ -544,7 +548,7 @@ gfarm_error_domain_add_map(struct gfarm_error_domain *domain,
 		GFARM_MALLOC_ARRAY(domain->map_to_gfarm,
 		    domain->domerror_number);
 		if (domain->map_to_gfarm == NULL) {
-			gflog_debug(GFARM_MSG_UNFIXED,
+			gflog_debug(GFARM_MSG_1000851,
 			    "gfarm_error_domain_add_map: no memory for %d",
 			    domain->domerror_number);
 			return (GFARM_ERR_NO_MEMORY);
@@ -600,7 +604,7 @@ gfarm_error_string(gfarm_error_t error)
 		return (errcode_string[GFARM_ERR_UNKNOWN]);
 	if (error < GFARM_ERR_NUMBER) {
 		if (error >= GFARM_ARRAY_LENGTH(errcode_string)) {
-			gflog_warning(GFARM_MSG_UNFIXED,
+			gflog_warning(GFARM_MSG_1000852,
 			    "gfarm_error_string: missing errcode_string: %d",
 			    error);
 			return (errcode_string[GFARM_ERR_UNKNOWN]);
@@ -669,7 +673,7 @@ gfarm_errno_to_error_initialize(void)
 			continue; /* a mapping is already registered */
 		if ((e = gfarm_error_domain_add_map(gfarm_errno_domain,
 		    map->unix_errno, map->gfarm_error)) != GFARM_ERR_NO_ERROR){
-			gflog_fatal(GFARM_MSG_UNFIXED,
+			gflog_fatal(GFARM_MSG_1000853,
 			    "libgfarm: initializing error map "
 			    "(%d -> %d): %s (%d)",
 			    map->unix_errno, map->gfarm_error,
