@@ -1,5 +1,5 @@
 /*
- * $Id: url.c 4457 2010-02-23 01:53:23Z ookuma $
+ * $Id: url.c 4924 2010-11-18 20:05:19Z n-soda $
  */
 
 #include <stddef.h>
@@ -99,8 +99,6 @@ gfarm_path_info_access(struct gfarm_path_info *pi, int mode)
  *	output: user/path/name
  */
 
-char GFARM_URL_PREFIX[] = "gfarm:";
-
 /*
  * Expand '~'.  Currently, '~/...' or '~username/...' is transformed
  * to '/username/...'.
@@ -287,13 +285,6 @@ gfarm_url_make_path_for_creation(const char *gfarm_url, char **canonic_pathp)
 	return (gfarm_canonical_path_for_creation(gfarm_url, canonic_pathp));
 }
 
-int
-gfarm_is_url(const char *gfarm_url)
-{
-	return (memcmp(gfarm_url, GFARM_URL_PREFIX, GFARM_URL_PREFIX_LENGTH)
-	    == 0);
-}
-
 /*
  * Translate a canonical path to a Gfarm URL.
  */
@@ -512,6 +503,15 @@ gfarm_url_prefix_add(const char *s)
 }
 #endif
 
+const char GFARM_URL_PREFIX[] = "gfarm:";
+
+int
+gfarm_is_url(const char *gfarm_url)
+{
+	return (memcmp(gfarm_url, GFARM_URL_PREFIX, GFARM_URL_PREFIX_LENGTH)
+	    == 0);
+}
+
 /*
  * Skip directory in the pathname.
  * We want traditional basename(3) here, rather than weird XPG one.
@@ -528,6 +528,77 @@ gfarm_path_dir_skip(const char *path)
 	return (base);
 }
 
+/* similar to dirname(3) in libc, but returns the result by malloc'ed memory */
+char *
+gfarm_url_dir(const char *pathname)
+{
+	char *parent, *top, *dir, *p;
+	int had_scheme = 0;
+	static const char dot[] = ".";
+
+	if (pathname[0] == '\0')
+		return (strdup(dot));
+	parent = strdup(pathname);
+	if (parent == NULL) {
+		gflog_debug(GFARM_MSG_1001463,
+			"allocation of dir failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
+		return (NULL);
+	}
+
+	top = dir = parent;
+	if (gfarm_is_url(dir)) {
+		had_scheme = 1;
+		dir += GFARM_URL_PREFIX_LENGTH;
+		top = dir;
+		if (dir[0] == '/') {
+			if (dir[1] != '/') {
+				top = dir;
+			} else {
+				dir += 2; /* skip "//" */
+				/* skip hostname:port */
+				while (dir[0] != '\0' && dir[0] != '/')
+					dir++;
+				top = dir - 1;
+				if (dir[0] == '/') {
+					for (p = dir + 1; *p == '/'; p++)
+						;
+					if (*p == '\0') {
+						dir[0] = '\0';
+						return (parent);
+					}
+				}
+			}
+		}
+		if (dir[0] == '\0')
+			return (parent);
+	}
+		
+	/* remove trailing '/' */
+	p = dir + strlen(dir) - 1;
+	while (p > dir && *p == '/')
+		--p;
+	p[1] = '\0';
+
+	p = (char *)gfarm_path_dir_skip(dir); /* UNCONST */
+	if (p == dir) { /* i.e. no slash */
+		if (had_scheme) {
+			dir[0] = '\0';
+			return (parent);
+		} else {
+			free(parent);
+			return (strdup(dot));
+		}
+	}
+	--p;
+
+	/* remove trailing '/' */
+	while (p > top && *p == '/')
+		--p;
+	p[1] = '\0';
+	return (parent);
+}
+
 /* similar to dirname(3) in libc, but returns the result by malloc'ed memory */
 char *
 gfarm_path_dir(const char *pathname)
@@ -565,28 +636,6 @@ gfarm_path_dir(const char *pathname)
 	return (dir);
 }
 
-char *
-gfarm_path_dirname(const char *pathname)
-{
-	char *parent = strdup(pathname), *b;
-
-	if (parent == NULL) {
-		gflog_debug(GFARM_MSG_1001464,
-			"allocation of path failed: %s",
-			gfarm_error_string(GFARM_ERR_NO_MEMORY));
-		return (NULL);
-	}
-
-	/* create parent directory canonic path */
-	for (b = (char *)gfarm_path_dir_skip(parent);
-	    b > parent && b[-1] == '/'; --b)
-		;
-	*b = '\0';
-
-	/* note that the root directory is '\0'. */
-	return (parent);
-}
-
 #if 0
 char *
 gfarm_url_make_localized_path(char *gfarm_url, char **abs_pathp)
