@@ -26,6 +26,7 @@
 
 #include "gfevent.h"
 #include "gfutil.h"
+#include "thrsubr.h"
 
 #include "tcputil.h"
 #include "gfarm_secure_session.h"
@@ -44,6 +45,8 @@ static int acceptorInitialized = 0;
 
 static pthread_mutex_t initiator_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t acceptor_mutex = PTHREAD_MUTEX_INITIALIZER;
+static const char initiatorDiag[] = "initiator";
+static const char acceptorDiag[] = "acceptor";
 
 /*
  * Communication option read from configuration file.
@@ -209,6 +212,7 @@ secSessionReadConfigFile(configFile, ssOptPtr)
 	/*
 	 * use default option.
 	 */
+	gflog_debug(GFARM_MSG_1000815, "fopen(%s) failed", configFile);
 	goto Done;
     }
 
@@ -556,6 +560,8 @@ allocSecSession(which)
 
     GFARM_MALLOC(ret);
     if (ret == NULL) {
+	gflog_debug(GFARM_MSG_1000816,
+		"allocation of gfarmSecSession failed");
 	return NULL;
     }
     (void)memset((void *)ret, 0, sizeof(gfarmSecSession));
@@ -572,6 +578,7 @@ allocSecSession(which)
 	default: {
 	    (void)free(ret);
 	    ret = NULL;
+	    gflog_debug(GFARM_MSG_1000817, "Invalid session type (%d)", which);
 	    return NULL;
 	}
     }
@@ -680,8 +687,9 @@ gfarmSecSessionInitializeAcceptor(configFile, usermapFile, majStatPtr, minStatPt
     int ret = 1;
     OM_uint32 majStat = GSS_S_COMPLETE;
     OM_uint32 minStat = GSS_S_COMPLETE;
+    static const char diag[] = "gfarmSecSessionInitializeAcceptor()";
 
-    pthread_mutex_lock(&acceptor_mutex);
+    gfarm_mutex_lock(&acceptor_mutex, diag, acceptorDiag);
     if (acceptorInitialized == 0) {
 	char confFile[PATH_MAX];
 
@@ -723,6 +731,8 @@ gfarmSecSessionInitializeAcceptor(configFile, usermapFile, majStatPtr, minStatPt
 	    majStat = GSS_S_FAILURE;
 	    minStat = GFSL_DEFAULT_MINOR_ERROR;
 	    ret = -1;
+	    gflog_debug(GFARM_MSG_1000818, "failed to read config file (%s)",
+		configFile);
 	    goto Done;
 	}
 
@@ -734,6 +744,8 @@ gfarmSecSessionInitializeAcceptor(configFile, usermapFile, majStatPtr, minStatPt
 	    majStat = GSS_S_FAILURE;
 	    minStat = GFSL_DEFAULT_MINOR_ERROR;
 	    ret = -1;
+	    gflog_debug(GFARM_MSG_1000819, "gfarmAuthInitialize(%s) failed",
+		usermapFile);
 	    goto Done;
 	}
 
@@ -748,7 +760,7 @@ gfarmSecSessionInitializeAcceptor(configFile, usermapFile, majStatPtr, minStatPt
 	    acceptorInitialized = 1;
 	}
     }
-    pthread_mutex_unlock(&acceptor_mutex);
+    gfarm_mutex_unlock(&acceptor_mutex, diag, acceptorDiag);
 
     if (majStatPtr != NULL) {
 	*majStatPtr = majStat;
@@ -764,11 +776,12 @@ gfarmSecSessionGetInitiatorInitialCredential(credPtr)
      gss_cred_id_t *credPtr;
 {
     int initiator_Initialized;
+    static const char diag[] = "gfarmSecSessionGetInitiatorInitialCredential()";
 
-    pthread_mutex_lock(&initiator_mutex);
+    gfarm_mutex_lock(&initiator_mutex, diag, initiatorDiag);
     *credPtr = initiatorInitialCred;
     initiator_Initialized = initiatorInitialized ? 1 : -1;
-    pthread_mutex_unlock(&initiator_mutex);
+    gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
 
     return initiator_Initialized;
 }
@@ -783,8 +796,9 @@ gfarmSecSessionInitializeInitiator(configFile, usermapFile, majStatPtr, minStatP
     int ret = 1;
     OM_uint32 majStat = GSS_S_COMPLETE;
     OM_uint32 minStat = GSS_S_COMPLETE;
+    static const char diag[] = "gfarmSecSessionInitializeInitiator()";
 
-    pthread_mutex_lock(&initiator_mutex);
+    gfarm_mutex_lock(&initiator_mutex, diag, initiatorDiag);
     if (initiatorInitialized == 0) {
 	char confFile[PATH_MAX];
 
@@ -826,6 +840,8 @@ gfarmSecSessionInitializeInitiator(configFile, usermapFile, majStatPtr, minStatP
 	    majStat = GSS_S_FAILURE;
 	    minStat = GFSL_DEFAULT_MINOR_ERROR;
 	    ret = -1;
+	    gflog_debug(GFARM_MSG_1000820, "failed to read config file (%s)",
+		configFile);
 	    goto Done;
 	}
 
@@ -855,7 +871,7 @@ gfarmSecSessionInitializeInitiator(configFile, usermapFile, majStatPtr, minStatP
 	    initiatorInitialized = 1;
 	}
     }
-    pthread_mutex_unlock(&initiator_mutex);
+    gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
 
     if (majStatPtr != NULL) {
 	*majStatPtr = majStat;
@@ -878,6 +894,7 @@ gfarmSecSessionInitializeBoth(iConfigFile, aConfigFile, usermapFile, majStatPtr,
     int ret = 1;
     OM_uint32 majStat = GSS_S_COMPLETE;
     OM_uint32 minStat = GSS_S_COMPLETE;
+    static const char diag[] = "gfarmSecSessionInitializeBoth()";
 
     /*
      * If the process's effective user is root (getuid()==0):
@@ -893,8 +910,8 @@ gfarmSecSessionInitializeBoth(iConfigFile, aConfigFile, usermapFile, majStatPtr,
      *	other GSSAPI implementation, a major change could be needed.
      */
 
-    pthread_mutex_lock(&initiator_mutex);
-    pthread_mutex_lock(&acceptor_mutex);
+    gfarm_mutex_lock(&initiator_mutex, diag, initiatorDiag);
+    gfarm_mutex_lock(&acceptor_mutex, diag, acceptorDiag);
     if (initiatorInitialized == 0 && acceptorInitialized == 0) {
 	char confFile[PATH_MAX];
 	char *confDir = NULL;
@@ -941,6 +958,8 @@ gfarmSecSessionInitializeBoth(iConfigFile, aConfigFile, usermapFile, majStatPtr,
 	    majStat = GSS_S_FAILURE;
 	    minStat = GFSL_DEFAULT_MINOR_ERROR;
 	    ret = -1;
+	    gflog_debug(GFARM_MSG_1000821,
+		"failed to read acceptor's config file (%s)", aConfigFile);
 	    goto Done;
 	}
 
@@ -959,6 +978,8 @@ gfarmSecSessionInitializeBoth(iConfigFile, aConfigFile, usermapFile, majStatPtr,
 	    majStat = GSS_S_FAILURE;
 	    minStat = GFSL_DEFAULT_MINOR_ERROR;
 	    ret = -1;
+	    gflog_debug(GFARM_MSG_1000822,
+		"failed to read initiator's config file (%s)", iConfigFile);
 	    goto Done;
 	}
 
@@ -970,6 +991,8 @@ gfarmSecSessionInitializeBoth(iConfigFile, aConfigFile, usermapFile, majStatPtr,
 	    majStat = GSS_S_FAILURE;
 	    minStat = GFSL_DEFAULT_MINOR_ERROR;
 	    ret = -1;
+	    gflog_debug(GFARM_MSG_1000823, "gfarmAuthInitialize(%s) failed",
+		usermapFile);
 	    goto Done;
 	}
 
@@ -988,8 +1011,8 @@ gfarmSecSessionInitializeBoth(iConfigFile, aConfigFile, usermapFile, majStatPtr,
 	    acceptorInitialized = 1;
 	}
     }
-    pthread_mutex_unlock(&acceptor_mutex);
-    pthread_mutex_unlock(&initiator_mutex);
+    gfarm_mutex_unlock(&acceptor_mutex, diag, acceptorDiag);
+    gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
 
     if (majStatPtr != NULL) {
 	*majStatPtr = majStat;
@@ -1004,7 +1027,9 @@ gfarmSecSessionInitializeBoth(iConfigFile, aConfigFile, usermapFile, majStatPtr,
 void
 gfarmSecSessionFinalizeInitiator()
 {
-    pthread_mutex_lock(&initiator_mutex);
+    static const char diag[] = "gfarmSecSessionFinalizeInitiator()";
+
+    gfarm_mutex_lock(&initiator_mutex, diag, initiatorDiag);
     if (initiatorInitialized == 1) {
 	if (initiatorInitialCred != GSS_C_NO_CREDENTIAL) {
 	    gfarmGssDeleteCredential(&initiatorInitialCred, NULL, NULL);
@@ -1012,14 +1037,16 @@ gfarmSecSessionFinalizeInitiator()
 	}
 	initiatorInitialized = 0;
     }
-    pthread_mutex_unlock(&initiator_mutex);
+    gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
 }
 
 
 void
 gfarmSecSessionFinalizeAcceptor()
 {
-    pthread_mutex_lock(&acceptor_mutex);
+    static const char diag[] = "gfarmSecSessionFinalizeAcceptor()";
+
+    gfarm_mutex_lock(&acceptor_mutex, diag, acceptorDiag);
     if (acceptorInitialized == 1) {
 	if (acceptorInitialCred != GSS_C_NO_CREDENTIAL) {
 	    gfarmGssDeleteCredential(&acceptorInitialCred, NULL, NULL);
@@ -1028,15 +1055,17 @@ gfarmSecSessionFinalizeAcceptor()
 	gfarmAuthFinalize();
 	acceptorInitialized = 0;
     }
-    pthread_mutex_unlock(&acceptor_mutex);
+    gfarm_mutex_unlock(&acceptor_mutex, diag, acceptorDiag);
 }
 
 
 void
 gfarmSecSessionFinalizeBoth()
 {
-    pthread_mutex_lock(&initiator_mutex);
-    pthread_mutex_lock(&acceptor_mutex);
+    static const char diag[] = "gfarmSecSessionFinalizeBoth()";
+
+    gfarm_mutex_lock(&initiator_mutex, diag, initiatorDiag);
+    gfarm_mutex_lock(&acceptor_mutex, diag, acceptorDiag);
     if (initiatorInitialized == 1 && acceptorInitialized == 1) {
 	if (acceptorInitialCred != GSS_C_NO_CREDENTIAL) {
 	    gfarmGssDeleteCredential(&acceptorInitialCred, NULL, NULL);
@@ -1047,8 +1076,8 @@ gfarmSecSessionFinalizeBoth()
 	acceptorInitialized = 0;
 	initiatorInitialized = 0;
     }
-    pthread_mutex_unlock(&acceptor_mutex);
-    pthread_mutex_unlock(&initiator_mutex);
+    gfarm_mutex_unlock(&acceptor_mutex, diag, acceptorDiag);
+    gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
 }
 
 
@@ -1080,9 +1109,11 @@ gfarmSecSessionAccept(fd, cred, ssOptPtr, majStatPtr, minStatPtr)
     unsigned int maxTransSize;
     unsigned int config;
 
-    pthread_mutex_lock(&acceptor_mutex);
+    static const char diag[] = "gfarmSecSessionAccept()";
+
+    gfarm_mutex_lock(&acceptor_mutex, diag, acceptorDiag);
     if (acceptorInitialized == 0) {
-	pthread_mutex_unlock(&acceptor_mutex);
+	gfarm_mutex_unlock(&acceptor_mutex, diag, acceptorDiag);
 	gflog_auth_error(GFARM_MSG_1000669,
 	    "gfarmSecSessionAccept(): not initialized");
 	goto Fail;
@@ -1090,14 +1121,15 @@ gfarmSecSessionAccept(fd, cred, ssOptPtr, majStatPtr, minStatPtr)
 
     ret = allocSecSession(GFARM_SS_ACCEPTOR);
     if (ret == NULL) {
-	pthread_mutex_unlock(&acceptor_mutex);
+	gfarm_mutex_unlock(&acceptor_mutex, diag, acceptorDiag);
 	gflog_auth_error(GFARM_MSG_1000670,
 	    "gfarmSecSessionAccept(): no memory");
 	goto Fail;
     }
 
     if (canonicSecSessionOpt(GFARM_SS_ACCEPTOR, ssOptPtr, &canOpt) < 0) {
-	pthread_mutex_unlock(&acceptor_mutex);
+	gfarm_mutex_unlock(&acceptor_mutex, diag, acceptorDiag);
+	gflog_debug(GFARM_MSG_1000824, "canonicSecSessionOpt() failed");
 	goto Fail;
     }
 
@@ -1105,7 +1137,8 @@ gfarmSecSessionAccept(fd, cred, ssOptPtr, majStatPtr, minStatPtr)
      * Get a peer information.
      */
     if (gfarmGetPeernameOfSocket(fd, &rPort, &peerName)) {
-	pthread_mutex_unlock(&acceptor_mutex);
+	gfarm_mutex_unlock(&acceptor_mutex, diag, acceptorDiag);
+	gflog_debug(GFARM_MSG_1000825, "gfarmGetPeernameOfSocket() failed");
 	goto Fail;
     }
 
@@ -1115,7 +1148,7 @@ gfarmSecSessionAccept(fd, cred, ssOptPtr, majStatPtr, minStatPtr)
     if (cred == GSS_C_NO_CREDENTIAL) {
 	cred = acceptorInitialCred;
     }
-    pthread_mutex_unlock(&acceptor_mutex);
+    gfarm_mutex_unlock(&acceptor_mutex, diag, acceptorDiag);
 
     /*
      * Phase 1: Accept a security context.
@@ -1123,6 +1156,9 @@ gfarmSecSessionAccept(fd, cred, ssOptPtr, majStatPtr, minStatPtr)
     if (gfarmGssAcceptSecurityContext(fd, cred, &sCtx,
 				      &majStat, &minStat,
 				      &initiatorName, &deleCred) < 0) {
+	gflog_debug(GFARM_MSG_1000826,
+		"gfarmGssAcceptSecurityContext failed(%u)(%u)",
+		majStat, minStat);
 	goto Fail;
     }
     if (initiatorName == GSS_C_NO_NAME ||
@@ -1182,6 +1218,9 @@ gfarmSecSessionAccept(fd, cred, ssOptPtr, majStatPtr, minStatPtr)
     if (negotiateConfigParam(fd, sCtx, GFARM_SS_ACCEPTOR, &canOpt,
 			     &qOp, &maxTransSize, &config,
 			     &majStat, &minStat) < 0) {
+	gflog_debug(GFARM_MSG_1000827,
+		"negotiateConfigParam() failed (%u)(%u)",
+		majStat, minStat);
 	goto Fail;
     }
 #if 0
@@ -1269,9 +1308,11 @@ secSessionInitiate(fd, acceptorName, cred, reqFlag, ssOptPtr, majStatPtr, minSta
     unsigned int maxTransSize;
     unsigned int config;
 
-    pthread_mutex_lock(&initiator_mutex);
+    static const char diag[] = "gfsl/secSessionInitiate()";
+
+    gfarm_mutex_lock(&initiator_mutex, diag, initiatorDiag);
     if (initiatorInitialized == 0) {
-	pthread_mutex_unlock(&initiator_mutex);
+	gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
 	gflog_auth_error(GFARM_MSG_1000674,
 	    "gfarm:secSessionInitiate(): not initialized");
 	goto Fail;
@@ -1279,14 +1320,16 @@ secSessionInitiate(fd, acceptorName, cred, reqFlag, ssOptPtr, majStatPtr, minSta
 
     ret = allocSecSession(GFARM_SS_INITIATOR);
     if (ret == NULL) {
-	pthread_mutex_unlock(&initiator_mutex);
+	gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
 	gflog_auth_error(GFARM_MSG_1000675,
 	    "gfarm:secSessionInitiate(): no memory");
 	goto Fail;
     }
 
     if (canonicSecSessionOpt(GFARM_SS_INITIATOR, ssOptPtr, &canOpt) < 0) {
-	pthread_mutex_unlock(&initiator_mutex);
+	gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
+	gflog_debug(GFARM_MSG_1000828,
+		"canonicSecSessionOpt() failed");
 	goto Fail;
     }
 
@@ -1294,7 +1337,9 @@ secSessionInitiate(fd, acceptorName, cred, reqFlag, ssOptPtr, majStatPtr, minSta
      * Get a peer information.
      */
     if (gfarmGetPeernameOfSocket(fd, &rPort, &peerName)) {
-	pthread_mutex_unlock(&initiator_mutex);
+	gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
+	gflog_debug(GFARM_MSG_1000829,
+		"gfarmGetPeernameOfSocket() failed");
 	goto Fail;
     }
 
@@ -1304,7 +1349,7 @@ secSessionInitiate(fd, acceptorName, cred, reqFlag, ssOptPtr, majStatPtr, minSta
     if (cred == GSS_C_NO_CREDENTIAL) {
 	cred = initiatorInitialCred;
     }
-    pthread_mutex_unlock(&initiator_mutex);
+    gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
 
     /*
      * Phase 1: Initiate a security context.
@@ -1312,6 +1357,9 @@ secSessionInitiate(fd, acceptorName, cred, reqFlag, ssOptPtr, majStatPtr, minSta
     if (gfarmGssInitiateSecurityContext(fd, acceptorName, cred, reqFlag, &sCtx,
 					&majStat, &minStat,
 					&acceptorNameResult) < 0) {
+	gflog_debug(GFARM_MSG_1000830,
+		"gfarmGssInitiateSecurityContext() failed (%u)(%u)",
+		majStat, minStat);
 	goto Fail;
     }
     if (acceptorNameResult == GSS_C_NO_NAME ||
@@ -1350,6 +1398,9 @@ secSessionInitiate(fd, acceptorName, cred, reqFlag, ssOptPtr, majStatPtr, minSta
     if (negotiateConfigParam(fd, sCtx, GFARM_SS_INITIATOR, &canOpt,
 			     &qOp, &maxTransSize, &config,
 			     &majStat, &minStat) < 0) {
+	gflog_debug(GFARM_MSG_1000831,
+		"negotiateConfigParam() failed (%u)(%u)",
+		majStat, minStat);
 	goto Fail;
     }
 #if 0
@@ -1439,6 +1490,9 @@ gfarmSecSessionInitiateByName(hostname, port, acceptorName, cred, reqFlag, ssOpt
 	if (minStatPtr != NULL) {
 	    *minStatPtr = GFSL_DEFAULT_MINOR_ERROR;
 	}
+	gflog_debug(GFARM_MSG_1000832,
+		"gfarmTCPConnectPortByHost(%s:%d) failed",
+		hostname, port);
 	return NULL;
     }
     return secSessionInitiate(fd, acceptorName, cred, reqFlag,
@@ -1508,12 +1562,14 @@ gfarmSecSessionDedicate(ssPtr)
 	    if (aePtr->authData.userAuth.gid != gid) {
 		if ((ret = setgid(aePtr->authData.userAuth.gid)) < 0) {
 		    ssPtr->gssLastStat = GSS_S_FAILURE;
+		    gflog_debug(GFARM_MSG_1000833, "setgid() failed");
 		    goto Done;
 		}
 	    }
 	    if (aePtr->authData.userAuth.uid != uid) {
 		if ((ret = setuid(aePtr->authData.userAuth.uid)) < 0) {
 		    ssPtr->gssLastStat = GSS_S_FAILURE;
+		    gflog_debug(GFARM_MSG_1000834, "setuid() failed");
 		    goto Done;
 		}
 	    }
@@ -1521,6 +1577,7 @@ gfarmSecSessionDedicate(ssPtr)
 	ret = 1;
     } else {
 	ssPtr->gssLastStat = GSS_S_FAILURE;
+	gflog_debug(GFARM_MSG_1000835, "gfarmAuthEntry is NULL");
     }
 
     Done:
@@ -1575,6 +1632,7 @@ gfarmSecSessionSendInt32(ssPtr, buf, n)
     GFARM_MALLOC_ARRAY(lBuf, n);
     if (lBuf == NULL) {
 	ssPtr->gssLastStat = GSS_S_FAILURE;
+	gflog_debug(GFARM_MSG_1000836, "allocation of buffer failed");
 	return ret;
     }
     for (i = 0; i < n; i++) {
@@ -1617,6 +1675,7 @@ gfarmSecSessionReceiveInt32(ssPtr, bufPtr, lenPtr)
 
     GFARM_MALLOC_ARRAY(retBuf, n);
     if (retBuf == NULL) {
+	gflog_debug(GFARM_MSG_1000837, "allocation of 'retBuf' failed");
 	goto Done;
     }
 
@@ -1655,6 +1714,7 @@ gfarmSecSessionSendInt16(ssPtr, buf, n)
     GFARM_MALLOC_ARRAY(lBuf, n);
     if (lBuf == NULL) {
 	ssPtr->gssLastStat = GSS_S_FAILURE;
+	gflog_debug(GFARM_MSG_1000838, "allocation of buffer failed");
 	return ret;
     }
     for (i = 0; i < n; i++) {
@@ -1697,6 +1757,7 @@ gfarmSecSessionReceiveInt16(ssPtr, bufPtr, lenPtr)
 
     GFARM_MALLOC_ARRAY(retBuf, n);
     if (retBuf == NULL) {
+	gflog_debug(GFARM_MSG_1000839, "allocation of 'retBuf' failed");
 	goto Done;
     }
 
@@ -2217,6 +2278,7 @@ secSessionInitiateRequest(q, fd, acceptorName, cred, reqFlag, ssOptPtr, continua
 	    "gfarm:secSessionInitiateRequest(): no memory");
     } else if (canonicSecSessionOpt(GFARM_SS_INITIATOR, ssOptPtr, &canOpt)< 0){
 	/* failure */;
+	gflog_debug(GFARM_MSG_1000840, "canonicSecSessionOpt() failed");
     } else {
 	state->q = q;
 	state->fd = fd;
@@ -2276,6 +2338,8 @@ secSessionInitiateRequest(q, fd, acceptorName, cred, reqFlag, ssOptPtr, continua
 	    majStat = GSS_S_FAILURE;
 	    minStat = GFSL_DEFAULT_MINOR_ERROR;
 	    gfarm_event_free(state->readable);
+	} else {
+	    gflog_debug(GFARM_MSG_1000841, "gfarm_fd_event_alloc() failed");
 	}
 
 	/* failure */
