@@ -4,6 +4,10 @@
 
 gfs_pio_test=$testbin/../gfs_pio_test/gfs_pio_test
 
+if $regress/bin/am_I_gfarmroot; then
+  exit $exit_unsupported
+fi
+
 trap 'gfrm -f $gftmp; rm -f $localtmp; exit $exit_trap' $trap_sigs
 
 if gfreg $data/1byte $gftmp && gfchmod 0444 $gftmp
