@@ -2,11 +2,11 @@
 
 . ./regress.conf
 
-trap 'gfrm $gftmp; rm -f $localtmp; exit $exit_trap' $trap_sigs
+trap 'gfrm $gftmp; gfxattr -r $gftop gfarm.ncopy; rm -f $localtmp; exit $exit_trap' $trap_sigs
 
 GFARM_FILE_RDWR=2	# from <gfarm/gfs.h>
 
-gfhost >$localtmp
+gfsched -w >$localtmp
 
 if [ `sed 2q $localtmp | wc -l` -ne 2 ]; then
     rm -f $localtmp
@@ -15,6 +15,9 @@ fi
 srchost=`sed -n 1p $localtmp`
 dsthost=`sed -n 2p $localtmp`
 
+# remove the effect of gfarm.ncopy in the root directory
+echo -n 1 | gfxattr -s $gftop gfarm.ncopy
+
 if gfreg -h $srchost $data/1byte $gftmp &&
    $testbin/file_busy $gftmp $GFARM_FILE_RDWR $dsthost 0 2>&1 |
 	fgrep 'file busy' >/dev/null
@@ -23,5 +26,6 @@ then
 fi
 
 gfrm $gftmp
+gfxattr -r $gftop gfarm.ncopy
 rm -f $localtmp
 exit $exit_code
