@@ -1,5 +1,5 @@
 /*
- * $Id: gfmd.c 4712 2010-06-09 06:26:23Z n-soda $
+ * $Id: gfmd.c 5018 2010-12-30 08:35:34Z tatebe $
  */
 
 #include <gfarm/gfarm_config.h>
@@ -46,7 +46,7 @@
 #include "gfj_client.h"
 #include "gfpath.h"
 
-#include "../gfsl/gfarm_auth.h"
+#include "gfarm_auth.h"	/* gfarmAuthInitialize(), gfarmAuthFinalize() */
 
 #include "subr.h"
 #include "thrpool.h"
@@ -108,6 +108,7 @@ gfm_server_protocol_extension_default(struct peer *peer,
 	gfarm_int32_t *requestp, gfarm_error_t *on_errorp)
 {
 	gflog_warning(GFARM_MSG_1000181, "unknown request: %d", request);
+	peer_record_protocol_error(peer);
 	return (GFARM_ERR_PROTOCOL);
 }
 
@@ -280,6 +281,9 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 		e = gfm_server_schedule_file_with_program(peer,
 		    from_client, skip);
 		break;
+	case GFM_PROTO_FGETATTRPLUS:
+		e = gfm_server_fgetattrplus(peer, from_client, skip);
+		break;
 	case GFM_PROTO_REMOVE:
 		e = gfm_server_remove(peer, from_client, skip);
 		break;
@@ -310,6 +314,9 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 	case GFM_PROTO_GETDIRENTSPLUS:
 		e = gfm_server_getdirentsplus(peer, from_client, skip);
 		break;
+	case GFM_PROTO_GETDIRENTSPLUSXATTR:
+		e = gfm_server_getdirentsplusxattr(peer, from_client, skip);
+		break;
 	case GFM_PROTO_REOPEN:
 		e = gfm_server_reopen(peer, from_client, skip,
 		    suspendedp);
@@ -409,8 +416,8 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 	case GFM_PROTO_REPLICA_ADDED2:
 		e = gfm_server_replica_added2(peer, from_client, skip);
 		break;
-	case GFM_PROTO_REPLICA_REMOVE:
-		e = gfm_server_replica_remove(peer, from_client, skip);
+	case GFM_PROTO_REPLICA_LOST:
+		e = gfm_server_replica_lost(peer, from_client, skip);
 		break;
 	case GFM_PROTO_REPLICA_ADD:
 		e = gfm_server_replica_add(peer, from_client, skip);
@@ -500,9 +507,11 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 		if (debug_mode)
 			gflog_debug(GFARM_MSG_1000182, "gfp_xdr_flush");
 		e2 = gfp_xdr_flush(peer_get_conn(peer));
-		if (e2 != GFARM_ERR_NO_ERROR)
+		if (e2 != GFARM_ERR_NO_ERROR) {
 			gflog_warning(GFARM_MSG_1000183, "protocol flush: %s",
 			    gfarm_error_string(e2));
+			peer_record_protocol_error(peer);
+		}
 		if (e == GFARM_ERR_NO_ERROR)
 			e = e2;
 	}
@@ -837,11 +846,13 @@ peer_authorize(struct peer *peer)
 		    id_type, username, hostname, &addr, auth_method);
 		giant_unlock();
 	} else {
-		gflog_warning(GFARM_MSG_1000187,
-		    "authorize: %s", gfarm_error_string(e));
+		gflog_warning(GFARM_MSG_1002474,
+		    "host %s: authorize: %s", hostname, gfarm_error_string(e));
+		free(hostname);
 	}
 	return (e);
 }
+ 
 
 void *
 try_auth(void *arg)
@@ -1090,6 +1101,7 @@ gfmd_modules_init_default(int table_size)
 	group_init();
 
 	/* filesystem */
+	inode_init_desired_number();
 	inode_init();
 	dir_entry_init();
 	file_copy_init();
@@ -1110,8 +1122,6 @@ void (*gfmd_modules_init)(int); /* intentionally remains uninitialized */
 int
 main(int argc, char **argv)
 {
-	extern char *optarg;
-	extern int optind;
 	gfarm_error_t e;
 	char *config_file = NULL, *port_number = NULL;
 	int syslog_level = -1;
@@ -1185,6 +1195,7 @@ main(int argc, char **argv)
 	}
 	if (syslog_level != -1)
 		gflog_set_priority_level(syslog_level);
+
 	if (port_number != NULL)
 		gfarm_metadb_server_port = strtol(port_number, NULL, 0);
 	sock = open_accepting_socket(gfarm_metadb_server_port);
@@ -1277,6 +1288,9 @@ main(int argc, char **argv)
 		    "create_detached_thread(resumer): %s",
 		    gfarm_error_string(e));
 
+	/* check and repair nlink: must be after db_thread */
+	inode_nlink_check();
+
 	accepting_loop(sock);
 
 	/*NOTREACHED*/
