@@ -17,6 +17,7 @@ void process_detach_peer(struct process *, struct peer *);
 struct user *process_get_user(struct process *);
 
 gfarm_error_t process_verify_fd(struct process *, int);
+gfarm_error_t process_record_desired_number(struct process *, int, int);
 gfarm_error_t process_get_file_inode(struct process *, int,
 	struct inode **);
 gfarm_error_t process_get_file_writable(struct process *, struct peer *, int);
@@ -47,6 +48,7 @@ struct file_opening {
 		struct opening_file {
 			struct peer *spool_opener;
 			struct host *spool_host;
+			int desired_replica_number;
 
 			/* only used by client initiated replication */
 			struct file_replicating *replicating;
@@ -80,6 +82,8 @@ struct file_opening {
 
 gfarm_error_t process_open_file(struct process *, struct inode *,
 	gfarm_int32_t, int, struct peer *, struct host *, gfarm_int32_t *);
+gfarm_error_t process_schedule_file(struct process *,
+	struct peer *, int, gfarm_int32_t *, struct host ***);
 gfarm_error_t process_reopen_file(struct process *,
 	struct peer *, struct host *, int,
 	gfarm_ino_t *, gfarm_uint64_t *, gfarm_int32_t *,
