@@ -1,5 +1,5 @@
 /*
- * $Id: quota.c 4554 2010-03-25 10:53:36Z tatebe $
+ * $Id: quota.c 4630 2010-04-06 12:42:54Z n-soda $
  */
 
 #include <stdlib.h>
@@ -405,11 +405,11 @@ quota_update_file_add_common(struct inode *inode, int quotacheck)
 		username = user_name(u);
 		groupname = group_name(g);
 		gflog_debug(GFARM_MSG_1000416,
-			    "<quota_add> "
-			    "ino=%"GFARM_PRId64"(gen=%"GFARM_PRId64"): "
-			    "size=%"GFARM_PRId64",ncopy=%"GFARM_PRId64","
-			    "user=%s,group=%s",
-			    inum, gen, size, ncopy, username, groupname);
+			    "<quota_add> ino=%lld(gen=%lld): "
+			    "size=%lld,ncopy=%lld,user=%s,group=%s",
+			    (unsigned long long)inum, (unsigned long long)gen,
+			    (unsigned long long)size, (unsigned long long)ncopy,
+			    username, groupname);
 	}
 }
 
@@ -600,11 +600,17 @@ quota_check_limits(struct user *u, struct group *g,
 
 	gettimeofday(&now, NULL);
 	if (u && is_exceeded(&now, user_quota(u),
-			    is_file_creating, is_replica_adding))
+			    is_file_creating, is_replica_adding)) {
+		gflog_debug(GFARM_MSG_1002051,
+			 "user_quota(%s) exceeded", user_name(u));
 		return (GFARM_ERR_DISK_QUOTA_EXCEEDED);
+	}
 	if (g && is_exceeded(&now, group_quota(g),
-			     is_file_creating, is_replica_adding))
+			     is_file_creating, is_replica_adding)) {
+		gflog_debug(GFARM_MSG_1002052,
+			 "group_quota(%s) exceeded", group_name(g));
 		return (GFARM_ERR_DISK_QUOTA_EXCEEDED);
+	}
 
 	return (GFARM_ERR_NO_ERROR);
 }
@@ -637,7 +643,8 @@ quota_group_remove(struct group *g)
 static gfarm_error_t
 quota_get_common(struct peer *peer, int from_client, int skip, int is_group)
 {
-	char *diag = is_group ? "quota_group_get" : "quota_user_get";
+	const char *diag = is_group ?
+	    "GFM_PROTO_QUOTA_GROUP_GET" : "GFM_PROTO_QUOTA_USER_GET";
 	gfarm_error_t e;
 	struct user *user = NULL, *peer_user = peer_get_user(peer);
 	struct group *group = NULL;
@@ -646,20 +653,27 @@ quota_get_common(struct peer *peer, int from_client, int skip, int is_group)
 	struct gfarm_quota_get_info qi;
 
 	e = gfm_server_get_request(peer, diag, "s", &name);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1002053,
+			"%s request failed: %s", diag, gfarm_error_string(e));
 		return (e);
+	}
 	if (skip) {
 		free(name);
 		return (GFARM_ERR_NO_ERROR);
 	}
 	if (!from_client || peer_user == NULL) {
 		e = GFARM_ERR_OPERATION_NOT_PERMITTED;
+		gflog_debug(GFARM_MSG_1002054,
+			    "%s: !from_client or invalid peer_user ", diag);
 		free(name);
 		return (gfm_server_put_reply(peer, diag, e, ""));
 	}
 
 	if (db_state != GFARM_ERR_NO_ERROR) {
 		free(name);
+		gflog_debug(GFARM_MSG_1002055, "db_quota is invalid: %s",
+			gfarm_error_string(db_state));
 		return (gfm_server_put_reply(peer, diag, db_state, ""));
 	}
 
@@ -685,7 +699,7 @@ quota_get_common(struct peer *peer, int from_client, int skip, int is_group)
 		if (strcmp(name, "") == 0) {
 			user = peer_user; /* permit not-admin */
 			free(name);
-			name = strdup(user_name(peer_user));
+			name = strdup_log(user_name(peer_user), diag);
 			if (name == NULL)
 				e = GFARM_ERR_NO_MEMORY;
 		} else if (strcmp(name, user_name(peer_user)) == 0)
@@ -699,6 +713,9 @@ quota_get_common(struct peer *peer, int from_client, int skip, int is_group)
 	}
 	if (e != GFARM_ERR_NO_ERROR) {
 		giant_unlock();
+		gflog_debug(GFARM_MSG_1002056,
+			    "%s: name=%s: %s", diag, name,
+			    gfarm_error_string(e));
 		free(name);
 		return (gfm_server_put_reply(peer, diag, e, ""));
 	}
@@ -708,6 +725,8 @@ quota_get_common(struct peer *peer, int from_client, int skip, int is_group)
 		q = user_quota(user);
 	if (!is_checked(q)) { /* quota is not enabled */
 		giant_unlock();
+		gflog_debug(GFARM_MSG_1002057,
+			    "%s: %s's quota is not enabled", diag, name);
 		free(name);
 		e = GFARM_ERR_NO_SUCH_OBJECT;
 		return (gfm_server_put_reply(peer, diag, e, ""));
@@ -750,7 +769,8 @@ quota_get_common(struct peer *peer, int from_client, int skip, int is_group)
 static gfarm_error_t
 quota_set_common(struct peer *peer, int from_client, int skip, int is_group)
 {
-	char *diag = is_group ? "quota_group_set" : "quota_user_set";
+	const char *diag = is_group ?
+	    "GFM_PROTO_QUOTA_GROUP_SET" : "GFM_PROTO_QUOTA_USER_SET";
 	gfarm_error_t e;
 	struct gfarm_quota_set_info qi;
 	struct quota *q;
@@ -768,14 +788,19 @@ quota_set_common(struct peer *peer, int from_client, int skip, int is_group)
 				   &qi.phy_space_hard,
 				   &qi.phy_num_soft,
 				   &qi.phy_num_hard);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1002058, "%s request failed: %s",
+			diag, gfarm_error_string(e));
 		return (e);
+	}
 	if (skip) {
 		free(qi.name);
 		return (GFARM_ERR_NO_ERROR);
 	}
 
 	if (db_state != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1002059, "db_quota is invalid: %s",
+			gfarm_error_string(db_state));
 		free(qi.name);
 		return (gfm_server_put_reply(peer, diag, db_state, ""));
 	}
@@ -783,11 +808,17 @@ quota_set_common(struct peer *peer, int from_client, int skip, int is_group)
 	giant_lock();
 	if (!from_client || peer_user == NULL || !user_is_admin(peer_user)) {
 		e = GFARM_ERR_OPERATION_NOT_PERMITTED;
+		gflog_debug(GFARM_MSG_1002060,
+			    "%s: !from_client or invalid peer_user"
+			    " or !user_is_admin", diag);
 		goto end;
 	} else if (is_group) {
 		group = group_lookup(qi.name);
 		if (!group_is_active(group)) {
 			e = GFARM_ERR_NO_SUCH_GROUP;
+			gflog_debug(GFARM_MSG_1002061,
+				    "%s: name=%s: %s",
+				    diag, qi.name, gfarm_error_string(e));
 			goto end;
 		}
 		q = group_quota(group);
@@ -795,6 +826,9 @@ quota_set_common(struct peer *peer, int from_client, int skip, int is_group)
 		user = user_lookup(qi.name);
 		if (!user_is_active(user)) {
 			e = GFARM_ERR_NO_SUCH_USER;
+			gflog_debug(GFARM_MSG_1002062,
+				    "%s: name=%s: %s",
+				    diag, qi.name, gfarm_error_string(e));
 			goto end;
 		}
 		q = user_quota(user);
@@ -853,23 +887,32 @@ gfm_server_quota_group_set(struct peer *peer, int from_client, int skip)
 gfarm_error_t
 gfm_server_quota_check(struct peer *peer, int from_client, int skip)
 {
-	char *diag = "quota_check";
+	static const char diag[] = "GFM_PROTO_QUOTA_CHECK";
 	gfarm_error_t e;
 	struct user *peer_user = peer_get_user(peer);
 
 	e = gfm_server_get_request(peer, diag, "");
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1002063,
+			"%s request failed: %s", diag, gfarm_error_string(e));
 		return (e);
+	}
 	if (skip)
 		return (GFARM_ERR_NO_ERROR);
 
-	if (db_state != GFARM_ERR_NO_ERROR)
+	if (db_state != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1002064, "db_quota is invalid: %s",
+			gfarm_error_string(db_state));
 		return (gfm_server_put_reply(peer, diag, db_state, ""));
+	}
 
 	giant_lock();
 	if (!from_client || peer_user == NULL || !user_is_admin(peer_user)) {
 		giant_unlock();
 		e = GFARM_ERR_OPERATION_NOT_PERMITTED;
+		gflog_debug(GFARM_MSG_1002065,
+			    "%s: !from_client or invalid peer_user"
+			    " or !user_is_admin", diag);
 		return (gfm_server_put_reply(peer, diag, e, ""));
 	}
 	/* zero clear and set true in is_checked */
