@@ -6,6 +6,9 @@ void giant_unlock(void);
 
 gfarm_error_t create_detached_thread(void *(*)(void *), void *);
 
+char *strdup_ck(const char *, const char *);
+char *strdup_log(const char *, const char *);
+
 int accmode_to_op(gfarm_uint32_t);
 
 struct peer;
