@@ -11,12 +11,6 @@
 #include <pthread.h>
 #include <errno.h>
 #include <sys/types.h> /* fd_set for "filetab.h" */
-#ifdef HAVE_SYS_XATTR_H
-#include <sys/xattr.h>
-#else
-#define XATTR_CREATE    0x1     /* set value, fail if attr already exists */
-#define XATTR_REPLACE   0x2     /* set value, fail if attr does not exist */
-#endif
 
 #include <gfarm/gfarm.h>
 
@@ -60,28 +54,33 @@ setxattr(int xmlMode, struct inode *inode, char *attrname,
 			"argument 'attrname' is invalid");
 		return GFARM_ERR_INVALID_ARGUMENT;
 	}
-	if ((flags & (XATTR_CREATE|XATTR_REPLACE))
-		== (XATTR_CREATE|XATTR_REPLACE)) {
+	if ((flags & (GFS_XATTR_CREATE|GFS_XATTR_REPLACE))
+		== (GFS_XATTR_CREATE|GFS_XATTR_REPLACE)) {
 		gflog_debug(GFARM_MSG_1002067,
 			"argument 'flags' is invalid");
 		return GFARM_ERR_INVALID_ARGUMENT;
 	}
-	if (flags & XATTR_REPLACE) {
-		if (!inode_xattr_isexists(inode, xmlMode, attrname)) {
+	if (flags & GFS_XATTR_REPLACE) {
+		e = inode_xattr_modify(inode, xmlMode, attrname, value, size);
+		if (e != GFARM_ERR_NO_ERROR) {
 			gflog_debug(GFARM_MSG_1002068,
-				"xattr does not exist");
-			return GFARM_ERR_NO_SUCH_OBJECT;
+			    "inode_xattr_modefy(%s): %s",
+			    attrname, gfarm_error_string(e));
+			return (e);
 		}
 	} else {
-		e = inode_xattr_add(inode, xmlMode, attrname);
+		e = inode_xattr_add(inode, xmlMode, attrname, value, size);
 		if (e == GFARM_ERR_NO_ERROR)
 			*addattr = 1;
-		else if (e != GFARM_ERR_ALREADY_EXISTS
-			|| (flags & XATTR_CREATE)) {
+		else if (e == GFARM_ERR_ALREADY_EXISTS &&
+		    (flags & GFS_XATTR_CREATE) == 0)
+			e = inode_xattr_modify(inode, xmlMode, attrname,
+					       value, size);
+		if (e != GFARM_ERR_NO_ERROR) {
 			gflog_debug(GFARM_MSG_1002069,
 				"inode_xattr_add() failed:%s",
 				gfarm_error_string(e));
-			return e;
+			return (e);
 		}
 	}
 
@@ -117,10 +116,11 @@ gfm_server_setxattr(struct peer *peer, int from_client, int skip, int xmlMode)
 		gflog_debug(GFARM_MSG_1002070,
 			"gfm_server_get_request() failure:%s",
 			gfarm_error_string(e));
-		goto quit;
+		return (e);
 	}
 	if (skip) {
-		goto quit;
+		free(attrname);
+		return (GFARM_ERR_NO_ERROR);
 	}
 	if (xmlMode) {
 		waitctx = &ctx;
@@ -185,24 +185,6 @@ quit:
 	return (gfm_server_put_reply(peer, diag, e, ""));
 }
 
-static gfarm_error_t
-getxattr(int xmlMode, struct inode *inode, char *attrname,
-	void **value, size_t *size, struct db_waitctx *waitctx)
-{
-	if (!isvalid_attrname(attrname)) {
-		gflog_debug(GFARM_MSG_1002077,
-			"argument 'attrname' is invalid");
-		return GFARM_ERR_INVALID_ARGUMENT;
-	}
-	if (!inode_xattr_isexists(inode, xmlMode, attrname)) {
-		gflog_debug(GFARM_MSG_1002078,
-			"xattr does not exist");
-		return GFARM_ERR_NO_SUCH_OBJECT;
-	}
-	return db_xattr_get(xmlMode, inode_get_number(inode),
-			attrname, value, size, waitctx);
-}
-
 gfarm_error_t
 gfm_server_getxattr(struct peer *peer, int from_client, int skip, int xmlMode)
 {
@@ -215,6 +197,7 @@ gfm_server_getxattr(struct peer *peer, int from_client, int skip, int xmlMode)
 	struct process *process;
 	gfarm_int32_t fd;
 	struct inode *inode;
+	int waitctx_initialized = 0, cached = 0;
 	struct db_waitctx waitctx;
 
 	e = gfm_server_get_request(peer, diag, "s", &attrname);
@@ -222,21 +205,21 @@ gfm_server_getxattr(struct peer *peer, int from_client, int skip, int xmlMode)
 		gflog_debug(GFARM_MSG_1002079,
 			"%s request failed: %s",
 			diag, gfarm_error_string(e));
-		goto quit;
+		return (e);
 	}
 	if (skip) {
-		goto quit;
+		free(attrname);
+		return (GFARM_ERR_NO_ERROR);
 	}
 #ifndef ENABLE_XMLATTR
 	if (xmlMode) {
 		gflog_debug(GFARM_MSG_1002080,
 			"operation is not supported(xmlMode)");
+		free(attrname);
 		e = GFARM_ERR_OPERATION_NOT_SUPPORTED;
-		goto quit;
+		return (gfm_server_put_reply(peer, diag, e, ""));
 	}
 #endif
-
-	db_waitctx_init(&waitctx);
 	giant_lock();
 	if ((process = peer_get_process(peer)) == NULL) {
 		e = GFARM_ERR_OPERATION_NOT_PERMITTED;
@@ -258,17 +241,32 @@ gfm_server_getxattr(struct peer *peer, int from_client, int skip, int xmlMode)
 		gflog_debug(GFARM_MSG_1002084,
 			"inode_access() failed: %s",
 			gfarm_error_string(e));
+	} else if (!isvalid_attrname(attrname)) {
+		e = GFARM_ERR_INVALID_ARGUMENT;
+		gflog_debug(GFARM_MSG_1002077,
+			"argument 'attrname' is invalid");
+	} else if ((e = inode_xattr_get_cache(inode, xmlMode, attrname,
+	    &value, &size)) != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1002510,
+		    "getxattr(%s): %s", attrname, gfarm_error_string(e));
+	} else if (value == NULL) { /* not cached */
+		db_waitctx_init(&waitctx);
+		waitctx_initialized = 1;
+		e = db_xattr_get(xmlMode, inode_get_number(inode),
+			attrname, &value, &size, &waitctx);
 	} else
-		e = getxattr(xmlMode, inode, attrname, &value, &size, &waitctx);
+		cached = 1;
 	giant_unlock();
-	if (e == GFARM_ERR_NO_ERROR)
+	if (e == GFARM_ERR_NO_ERROR && !cached)
 		e = dbq_waitret(&waitctx);
-	db_waitctx_fini(&waitctx);
-quit:
+	if (waitctx_initialized)
+		db_waitctx_fini(&waitctx);
+
 	e = gfm_server_put_reply(peer, diag, e, "b", size, value);
 	free(attrname);
-	free(value);
-	return e;
+	if (value != NULL)
+		free(value);
+	return (e);
 }
 
 gfarm_error_t
@@ -283,9 +281,8 @@ gfm_server_listxattr(struct peer *peer, int from_client, int skip, int xmlMode)
 	gfarm_int32_t fd;
 	struct inode *inode;
 
-	if (skip) {
-		return GFARM_ERR_NO_ERROR;
-	}
+	if (skip)
+		return (GFARM_ERR_NO_ERROR);
 #ifndef ENABLE_XMLATTR
 	if (xmlMode)
 		return gfm_server_put_reply(peer, diag,
@@ -362,20 +359,21 @@ gfm_server_removexattr(struct peer *peer, int from_client, int skip,
 		gflog_debug(GFARM_MSG_1002090,
 			"%s request failure",
 			diag);
-		goto quit;
+		return (e);
 	}
 	if (skip) {
-		goto quit;
+		free(attrname);
+		return (GFARM_ERR_NO_ERROR);
 	}
 #ifndef ENABLE_XMLATTR
 	if (xmlMode) {
 		gflog_debug(GFARM_MSG_1002091,
 			"operation is not supported(xmlMode)");
+		free(attrname);
 		e = GFARM_ERR_OPERATION_NOT_SUPPORTED;
-		goto quit;
+		return (gfm_server_put_reply(peer, diag, e, ""));
 	}
 #endif
-
 	giant_lock();
 	if ((process = peer_get_process(peer)) == NULL) {
 		e = GFARM_ERR_OPERATION_NOT_PERMITTED;
@@ -400,11 +398,12 @@ gfm_server_removexattr(struct peer *peer, int from_client, int skip,
 	} else
 		e = removexattr(xmlMode, inode, attrname);
 	giant_unlock();
-quit:
+
 	free(attrname);
 	return (gfm_server_put_reply(peer, diag, e, ""));
 }
 
+
 #ifdef ENABLE_XMLATTR
 /*
  * These parameters must be smaller than DBQ_SIZE
@@ -1086,17 +1085,20 @@ gfm_server_findxmlattr(struct peer *peer, int from_client, int skip)
 
 	e = gfm_server_get_request(peer, diag,
 			"siiss", &expr, &depth, &nalloc, &ck_path, &ck_name);
-#ifdef ENABLE_XMLATTR
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1002110,
 			"%s request failed: %s",
 			diag, gfarm_error_string(e));
-		goto quit;
+		return (e);
 	}
 	if (skip) {
-		goto quit;
+		free(expr);
+		free(ck_path);
+		free(ck_name);
+		return (GFARM_ERR_NO_ERROR);
 	}
 
+#ifdef ENABLE_XMLATTR
 	if ((ctxp = gfs_xmlattr_ctx_alloc(nalloc)) == NULL) {
 		e = GFARM_ERR_NO_MEMORY;
 		gflog_debug(GFARM_MSG_1002111,
